@@ -2,13 +2,15 @@ from .cusp import CuspCorrection, ElectronicAsymptotic
 from .distbasis import DistanceBasis
 from .gto import GTOBasis, GTOShell
 from .molorb import MolecularOrbital
-from .omni import OmniSchNet
+from .omni import Backflow, Jastrow, OmniSchNet
 from .paulinet import PauliNet
 from .schnet import ElectronicSchNet, SubnetFactory
 
 __all__ = [
     'PauliNet',
     'OmniSchNet',
+    'Jastrow',
+    'Backflow',
     'ElectronicSchNet',
     'SubnetFactory',
     'DistanceBasis',
