@@ -6,18 +6,17 @@ from torch import nn
 
 from deepqmc import Molecule
 from deepqmc.physics import pairwise_diffs, pairwise_distance
+from deepqmc.plugins import PLUGINS
 from deepqmc.torchext import sloglindet, triu_flat
 from deepqmc.wf import WaveFunction
 
 from .cusp import CuspCorrection, ElectronicAsymptotic
-from .distbasis import DistanceBasis
 from .gto import GTOBasis
 from .molorb import MolecularOrbital
-from .omni import OmniSchNet, SubnetFactory
+from .omni import OmniSchNet
 from .pyscfext import pyscf_from_mol
-from .schnet import ElectronicSchNet
 
-__version__ = '0.1.0'
+__version__ = '0.2.0'
 __all__ = ['PauliNet']
 
 log = logging.getLogger(__name__)
@@ -36,7 +35,7 @@ def eval_log_slater(xs):
 
 
 class PauliNet(WaveFunction):
-    r"""Implements the PauliNet ansatz from [Hermann19]_.
+    r"""Implements the PauliNet ansatz from [HermannNC20]_.
 
     Derived from :class:`WaveFunction`. This constructor provides a fully
     flexible low-level interface. See the alternative constructors for the
@@ -44,97 +43,90 @@ class PauliNet(WaveFunction):
 
     The PauliNet ansatz combines a multireference Slater determinant expansion,
     Gaussian-type cusp-corrected molecular orbitals (:class:`MolecularOrbital`),
-    electronic cusp Jastrow factor (:class:`ElectronicAsymptotic`) and many-body
-    Jastrow factor and backflow transformation represented by neural networks
-    that use featurized particle distances,
-    :math:`\mathbf e(|\mathbf r-\mathbf r'|)` (:class:`DistanceBasis`), as input,
+    electronic cusp Jastrow factor (:class:`ElectronicAsymptotic`) and
+    trainable Jastrow factor and backflow transformation (:class:`OmniSchNet`).
 
     .. math::
-        \psi_{\boldsymbol\theta}(\mathbf r)
-          =\mathrm e^{\gamma(\mathbf r)+J_{\boldsymbol\theta}(\mathbf r)}
-          \sum_{pq} c_p
-          \det[\tilde\varphi_{\boldsymbol\theta,q{\mu_p}i}^\uparrow(\mathbf r)]
-          \det[\tilde\varphi_{\boldsymbol\theta,q{\mu_p}i}^\downarrow(\mathbf r)] \\
-        \tilde\varphi_{\boldsymbol\theta,q\mu i}(\mathbf r)
-          :=\big(1+2\tanh(\kappa_{\boldsymbol\theta,q\mu i}(\mathbf r))\big)
-          \varphi_\mu(\mathbf r_i)
+        \psi_{\boldsymbol\theta}(\mathbf r) =\mathrm e^{\gamma(\mathbf
+        r)+J_{\boldsymbol\theta}(\mathbf x_i^{(L)})}
+        \sum_{pq}\frac{c_p}{N_\text{bf}}
+        \det[\tilde{\boldsymbol\varphi}_{\boldsymbol\theta,pq}^\uparrow(\mathbf
+        r)]
+        \det[\tilde{\boldsymbol\varphi}_{\boldsymbol\theta,pq}^\downarrow(\mathbf
+        r)] \\ \tilde\varphi_{\boldsymbol\theta,pq\nu i}(\mathbf r)
+        :=\varphi_{\mu_{p\nu}}(\mathbf r_i)\star\big(\mathbf f_{*_1}(\mathbf
+        x_i^{(L)})\big)_{*_2} \\ (*_1,*_2)=\begin{cases} (q,\mu_{p\nu}) &
+        \text{orbital-type backflow} \\ (pq,\nu) & \text{determinant-type
+        backflow} \end{cases}
 
-    Here, :math:`c_p,\mu_p` define the multideterminant expansion,
-    :math:`\varphi_\mu(\mathbf r)` are the baseline
-    single-electron molecular orbitals, :math:`J_{\boldsymbol\theta}(\mathbf r)`
-    is the permutation-invariant deep Jastrow factor,
-    :math:`\kappa_{\boldsymbol\theta,q\mu i}(\mathbf r)` is the :math:`q`-th
-    channel of the permutation-equivariant deep backflow, and :math:`\gamma`
+    Here, :math:`\mathbf x_i^{(L)}` are the permutation-equivariant electron
+    embeddings obtained from a graph neural network, :math:`\mu_{p\nu}`
+    defines the electron configurations with the corresponding linear coefficients
+    :math:`c_p`, :math:`\varphi_\mu(\mathbf r)` are
+    the baseline single-electron molecular orbitals,
+    :math:`J_{\boldsymbol\theta}` is the permutation-invariant deep Jastrow
+    factor, :math:`\mathbf f_{\boldsymbol\theta}` is the
+    permutation-equivariant deep backflow, and :math:`\gamma`
     enforces correct electronic cusp conditions.
+    Based on the type of the backflow, the backflow is applied either
+    orbital-wise or determinant-wise, and :math:`\star` can be either multiplication
+    or addition.
+    :math:`N_\text{bf}` different backflows indexed by :math:`q` are used with
+    each electron configuration.
 
     Args:
         mol (:class:`~deepqmc.Molecule`): molecule whose wave function is represented
         basis (:class:`~deepqmc.wf.paulinet.GTOBasis`): basis for the molecular orbitals
-        jastrow_factory (callable): constructor for a Jastrow factor,
-            :math:`(M,\dim(\mathbf e),N^\uparrow,N^\downarrow)`
-            :math:`\rightarrow(\mathbf e_{ij},\mathbf e_{iI})\rightarrow J`
-        backflow_factory (callable): constructor for a backflow,
-            :math:`(M,\dim(\mathbf e),N^\uparrow,N^\downarrow,N_\text{orb},C)`
-            :math:`\rightarrow(\mathbf e_{ij},\mathbf e_{iI})`
-            :math:`\rightarrow\kappa_{q\mu i}`
-        omni_factory (callable): constructor for a combined Jastrow factor and backflow,
-            with interface identical to :class:`~deepqmc.wf.paulinet.OmniSchNet`
         n_configurations (int): number of electron configurations
         n_orbitals (int): number of distinct molecular orbitals used across all
             configurations if given, otherwise the larger of the number of spin-up
             and spin-down electrons
-        mo_factory (callable): passed to :class:`~deepqmc.wf.paulinet.MolecularOrbital`
-            as ``net_factory``
         cusp_correction (bool): whether nuclear cusp correction is used
         cusp_electrons (bool): whether electronic cusp function is used
-        dist_feat_dim (int): :math:`\dim(\mathbf e)`, number of distance features
-        dist_feat_cutoff (float, a.u.): distance at which distance features
-            go to zero
-        backflow_channels (int): :math:`C`, number of backflow channels
-        omni_kwargs: arguments passed to :class:`~deepqmc.wf.paulinet.OmniSchNet`
+        backflow_type (str): ``'orbital'`` for orbital-type backflow, ``'det'``
+            for determinant-type backflow
+        backflow_channels (int): :math:`N_\text{bf}`, number of backflow channels
+        backflow_transform (str): specifies the :math:`\star` operation:
+
+            - ``'mult'`` -- :math:`x\star y:=x\big(1+2\tanh(y)\big)`
+        omni_factory (callable): constructor for combined a Jastrow and backflow,
+            :math:`(M,N^\uparrow,N^\downarrow,N_\text{orb},N_\text{bf})`
+            :math:`\rightarrow(r_{ij},R_{iI})\rightarrow (J,f_{q\mu i})`
+        omni_kwargs: extra arguments passed to ``omni_factory``
 
     Attributes:
-        jastrow: :class:`torch.nn.Module` representing the Jastrow factor
-        backflow: :class:`torch.nn.Module` representing the backflow transformation
+        omni: :class:`torch.nn.Module` representing Jastrow and backflow
         conf_coeff: :class:`torch.nn.Linear` with no bias that represents
             the multireference coefficients :math:`c_p` via its :attr:`weight`
             variable of shape :math:`(1,N_\text{det})`
     """
 
+    OMNI_FACTORIES = {'omni_schnet': OmniSchNet}
+
     def __init__(
         self,
         mol,
         basis,
-        jastrow_factory=None,
-        backflow_factory=None,
-        omni_factory=None,
         n_configurations=1,
         n_orbitals=None,
-        mo_factory=None,
         return_log=True,
         use_sloglindet='training',
         *,
         cusp_correction=True,
         cusp_electrons=True,
-        dist_feat_dim=32,
-        dist_feat_cutoff=10.0,
         backflow_type='orbital',
         backflow_channels=1,
         backflow_transform='mult',
         rc_scaling=1.0,
         cusp_alpha=10.0,
         freeze_embed=False,
+        omni_factory='omni_schnet',
         omni_kwargs=None,
     ):
         assert use_sloglindet in {'never', 'training', 'always'}
         assert return_log or use_sloglindet == 'never'
         super().__init__(mol)
         n_up, n_down = self.n_up, self.n_down
-        self.dist_basis = (
-            DistanceBasis(dist_feat_dim, cutoff=dist_feat_cutoff, envelope='nocusp')
-            if mo_factory or jastrow_factory or backflow_factory or omni_factory
-            else None
-        )
         n_orbitals = n_orbitals or max(n_up, n_down)
         confs = [list(range(n_up)) + list(range(n_down))] + [
             sum((torch.randperm(n_orbitals)[:n].tolist() for n in (n_up, n_down)), [])
@@ -150,8 +142,6 @@ class PauliNet(WaveFunction):
             mol,
             basis,
             n_orbitals,
-            net_factory=mo_factory,
-            dist_feat_dim=dist_feat_dim,
             cusp_correction=cusp_correction,
             rc_scaling=rc_scaling,
         )
@@ -160,11 +150,6 @@ class PauliNet(WaveFunction):
             if cusp_electrons
             else (None, None)
         )
-        self.jastrow = (
-            jastrow_factory(len(mol), dist_feat_dim, n_up, n_down)
-            if jastrow_factory
-            else None
-        )
         backflow_spec = {
             'orbital': [n_orbitals, backflow_channels],
             'det': [max(n_up, n_down), len(self.confs) * backflow_channels],
@@ -173,27 +158,24 @@ class PauliNet(WaveFunction):
             backflow_spec[1] *= 2
         self.backflow_type = backflow_type
         self.backflow_transform = backflow_transform
-        self.backflow = (
-            backflow_factory(len(mol), dist_feat_dim, n_up, n_down, *backflow_spec)
-            if backflow_factory
+        if 'paulinet.omni_factory' in PLUGINS:
+            log.info('Using a plugin for paulinet.omni_factory')
+            omni_factory = PLUGINS['paulinet.omni_factory']
+        elif isinstance(omni_factory, str):
+            if omni_kwargs:
+                omni_kwargs = omni_kwargs[omni_factory]
+            omni_factory = self.OMNI_FACTORIES[omni_factory]
+        self.omni = (
+            omni_factory(
+                len(mol.coords), n_up, n_down, *backflow_spec, **(omni_kwargs or {})
+            )
+            if omni_factory
             else None
         )
-        self.r_backflow = None
-        if omni_factory:
-            assert not backflow_factory and not jastrow_factory
-            self.omni = omni_factory(
-                mol, dist_feat_dim, n_up, n_down, *backflow_spec, **(omni_kwargs or {})
-            )
-            self.backflow = self.omni.forward_backflow
-            self.r_backflow = self.omni.forward_r_backflow
-            self.jastrow = self.omni.forward_jastrow
-        else:
-            self.omni = None
         self.return_log = return_log
         if freeze_embed:
             self.requires_grad_embeddings_(False)
         self.n_determinants = len(self.confs) * backflow_channels
-        self.n_backflows = 0 if not self.backflow else backflow_spec[1]
         if n_up <= 1 or n_down <= 1:
             self.use_sloglindet = 'never'
             log.warning(
@@ -221,12 +203,19 @@ class PauliNet(WaveFunction):
 
     @classmethod
     def DEFAULTS(cls):
+        from .omni import Backflow, Jastrow
+        from .schnet import ElectronicSchNet, SubnetFactory
+
         return {
             (cls.from_hf, 'kwargs'): cls.from_pyscf,
             (cls.from_pyscf, 'kwargs'): cls,
-            (cls, 'omni_kwargs'): OmniSchNet,
+            (cls, 'omni_kwargs'): cls.OMNI_FACTORIES,
             (OmniSchNet, 'schnet_kwargs'): ElectronicSchNet,
+            (OmniSchNet, 'mf_schnet_kwargs'): (ElectronicSchNet, ['version']),
             (OmniSchNet, 'subnet_kwargs'): SubnetFactory,
+            (OmniSchNet, 'mf_subnet_kwargs'): SubnetFactory,
+            (OmniSchNet, 'jastrow_kwargs'): Jastrow,
+            (OmniSchNet, 'backflow_kwargs'): Backflow,
         }
 
     @classmethod
@@ -266,7 +255,7 @@ class PauliNet(WaveFunction):
         if hasattr(mf, 'fcisolver'):
             if conf_limit:
                 conf_cutoff = max(
-                    np.sort(abs(mf.ci.flatten()))[-conf_limit:][0] - 1e-10, conf_cutoff,
+                    np.sort(abs(mf.ci.flatten()))[-conf_limit:][0] - 1e-10, conf_cutoff
                 )
             for tol in [conf_cutoff, conf_cutoff + 2e-10]:
                 conf_coeff, *confs = zip(
@@ -301,7 +290,7 @@ class PauliNet(WaveFunction):
             mf.mol.spin,
         )
         basis = GTOBasis.from_pyscf(mf.mol)
-        wf = cls(mol, basis, **{'omni_factory': OmniSchNet, **kwargs})
+        wf = cls(mol, basis, **kwargs)
         if init_weights:
             wf.mo.init_from_pyscf(mf, freeze_mos=freeze_mos)
             if confs is not None:
@@ -362,33 +351,23 @@ class PauliNet(WaveFunction):
         n_atoms = len(self.mol)
         coords = self.mol.coords
         diffs_nuc = pairwise_diffs(torch.cat([coords, rs.flatten(end_dim=1)]), coords)
-        edges_nuc = (
-            self.dist_basis(diffs_nuc[:, :, 3].sqrt())
-            if self.jastrow or self.mo.net
-            else None
-        )
-        if self.r_backflow or self.backflow or self.cusp_same or self.jastrow:
-            dists_elec = pairwise_distance(rs, rs)
-        if self.r_backflow or self.backflow or self.jastrow:
-            edges_nuc = edges_nuc[n_atoms:].view(batch_dim, n_elec, n_atoms, -1)
-            edges = self.dist_basis(dists_elec), edges_nuc
-        if self.r_backflow:
-            rs_flowed = self.r_backflow(rs, *edges)
-            diffs_nuc = pairwise_diffs(
-                torch.cat([coords, rs_flowed.flatten(end_dim=1)]), coords
+        dists_elec = pairwise_distance(rs, rs)
+        if self.omni:
+            dists_nuc = (
+                diffs_nuc[n_atoms:, :, 3].sqrt().view(batch_dim, n_elec, n_atoms)
             )
-        xs = self.mo(diffs_nuc, edges_nuc)
+        xs = self.mo(diffs_nuc)
         # get orbitals as [bs, 1, i, mu]
         xs = xs.view(batch_dim, 1, n_elec, -1)
-        if self.backflow:
-            fs = self.backflow(*edges)  # [bs, q, i, mu/nu]
-            if self.backflow_type == 'orbital':
-                xs = self._backflow_op(xs, fs)
+        # get jastrow J and backflow fs (as [bs, q, i, mu/nu])
+        J, fs = self.omni(dists_nuc, dists_elec) if self.omni else (None, None)
+        if fs is not None and self.backflow_type == 'orbital':
+            xs = self._backflow_op(xs, fs)
         # form dets as [bs, q, p, i, nu]
         conf_up, conf_down = self.confs[:, : self.n_up], self.confs[:, self.n_up :]
         det_up = xs[:, :, : self.n_up, conf_up].transpose(-3, -2)
         det_down = xs[:, :, self.n_up :, conf_down].transpose(-3, -2)
-        if self.backflow and self.backflow_type == 'det':
+        if fs is not None and self.backflow_type == 'det':
             n_conf = len(self.confs)
             fs = fs.unflatten(1, ((None, fs.shape[1] // n_conf), (None, n_conf)))
             det_up = self._backflow_op(det_up, fs[..., : self.n_up, : self.n_up])
@@ -437,9 +416,6 @@ class PauliNet(WaveFunction):
                 if self.return_log
                 else psi * torch.exp(cusp_same + cusp_anti)
             )
-        if self.jastrow:
-            J = self.jastrow(*edges)
+        if J is not None:
             psi = psi + J if self.return_log else psi * torch.exp(J)
-        if self.omni:
-            self.omni.forward_close()
         return (psi, sign) if self.return_log else psi
